// created on 11/19/2005 at 1:26 AM by Aydemir Ulas Sahin

using System;
using System.Data;
using System.IO;
using System.Collections;
using GLib;
using Gtk;
using Mono.Unix;
using Sql;

namespace Banshee
{	
	public class ArtistAlbumBrowser : VBox
	{	
        private Gtk.HBox main_box = new Gtk.HBox();
        private Gtk.ScrolledWindow artist_scroller = new Gtk.ScrolledWindow();
        private Gtk.ScrolledWindow album_scroller = new Gtk.ScrolledWindow();
        private Gtk.TreeView artist_tree = new Gtk.TreeView();
        private Gtk.TreeView album_tree = new Gtk.TreeView();
        
        private ListStore artist_store;
        private ListStore album_store;
        
        private string artist_selected = "none";
        private string album_selected = "none";
        
        private int artist_selection_index = 0;
        private int album_selection_index = 0;
        
        private Statement query;
        private string library_query;
        
        private int [] ResetSelectPath = {0};
        
        public event EventHandler BrowserClicked;
        
		public ArtistAlbumBrowser() : base()
		{
			BuildUI();
			InitBrowser();
		}
		
		public string Query
        {
            get {
                return library_query;
            	}
             
            set {
                 library_query = value;
             	}
        }
		
		private void BuildUI()
		{
		 	artist_scroller.HscrollbarPolicy = PolicyType.Automatic;
		 	artist_scroller.VscrollbarPolicy = PolicyType.Automatic;
		 	artist_scroller.BorderWidth = 3;
		 	
			album_scroller.HscrollbarPolicy = PolicyType.Automatic;			
			album_scroller.VscrollbarPolicy = PolicyType.Automatic;
			album_scroller.BorderWidth = 3;
			
			artist_tree.HeadersVisible = true;
			artist_tree.AppendColumn(Catalog.GetString("Artist"), new CellRendererText(), "text", 0);
			artist_store = CreateStore();
			artist_tree.Model = artist_store;
			artist_tree.CursorChanged += OnArtistCursorChanged;
			artist_scroller.Add(artist_tree);
			
			album_tree.HeadersVisible = true;
			album_tree.AppendColumn(Catalog.GetString("Album"), new CellRendererText(), "text", 0);
			album_store = CreateStore();
			album_tree.Model = album_store;
			album_tree.CursorChanged += OnAlbumCursorChanged;
			album_scroller.Add(album_tree);
						
			main_box.PackStart(artist_scroller, true, true, 0);
			main_box.PackStart(album_scroller, true, true, 0);					
			
			PackStart(main_box, true, true, 0);
			Spacing = 5;
			
		}
		
		private void InitBrowser()
		{
			query = new Select("Tracks", new Statement("DISTINCT Artist"));
			PopulateArtistStore();
			PopulateAlbumStore();
			
			artist_tree.Selection.SelectPath(new TreePath(ResetSelectPath));
			album_tree.Selection.SelectPath(new TreePath(ResetSelectPath));
			
			this.ShowAll();
		}
		 
		private ListStore CreateStore()
        {
            ListStore store = new ListStore(typeof(string));
			
			//We dont need a sort for this view but in case anyone asks...
			/*		
            store.SetSortFunc(1, delegate(TreeModel model, TreeIter a, TreeIter b) {
                try {
                    String a_string = (String)store.GetValue(a, 1);
                    String b_string = (String)store.GetValue(b, 1);
                    return String.Compare(a_string, b_string);
                } catch(Exception) {
                    return 0;
                }
            });
            
            store.SetSortColumnId(1, SortType.Descending);
            */
            return store;
        }
        
        private void PopulateArtistStore()
        {
            artist_store.Clear();
            ArrayList artistlist = new ArrayList();
            try 
            {
                IDataReader reader = Core.Library.Db.Query(query);
                while(reader.Read())
                {
                    artistlist.Add((reader[0] as string));
                }
               
            } catch(Exception) {
            
            }
            
            AddEntry("All", true, artist_store);
            foreach(string entry in artistlist) {
                AddEntry(entry, false, artist_store);
            }
            
            ResetArtistList();
        }
        
        private void PopulateAlbumStore()
        {
        	album_store.Clear();
        	ArrayList albumlist = new ArrayList();
        	if (artist_selection_index == 0)
        	{
        		query = new Select("Tracks", new Statement("DISTINCT AlbumTitle, Artist"));
        	}
        	else
        	{
        		query = new Select("Tracks", new Statement("DISTINCT AlbumTitle, Artist"))
        		+ new Where(new Compare("Artist", Op.EqualTo, artist_selected));
        	} 
            
            try 
            {
                IDataReader reader = Core.Library.Db.Query(query);
                while(reader.Read())
                {              
                    albumlist.Add((reader[0] as string));
                }
               
            } catch(Exception) {
            
            }
            
            AddEntry("All", true, album_store);
            foreach(string entry in albumlist) {
                AddEntry(entry, false, album_store);
            }
            
            ResetAlbumList();
        }
        
        private void ResetAlbumList()
        {
            album_tree.Selection.SelectPath(new TreePath(ResetSelectPath));
           	TreePath[] treeIndex = album_tree.Selection.GetSelectedRows();
			album_selection_index = treeIndex[0].Indices[0];
        }
        
        private void ResetArtistList()
        {
            artist_tree.Selection.SelectPath(new TreePath(ResetSelectPath));
            TreePath[] treeIndex = artist_tree.Selection.GetSelectedRows();
			artist_selection_index = treeIndex[0].Indices[0];
        }
		
		private void AddEntry(string entry, bool prepend,ListStore store)
        {   
            TreeIter iter = prepend ? store.Insert(0) : store.Append();
            store.SetValue(iter, 0, entry);
        }
        
        private void OnArtistCursorChanged(object o, EventArgs args)
        {
            TreeIter iter;
            
            if(!artist_tree.Selection.GetSelected(out iter)) {
                return;
            }
            
            object artist_name = artist_store.GetValue(iter, 0);
            artist_selected = (artist_name as string);
            
            TreePath[] treeIndex = artist_tree.Selection.GetSelectedRows();
			artist_selection_index = treeIndex[0].Indices[0];
            
            if(artist_name == null) {
                return;
            }
            
            PopulateAlbumStore();
            RunQuery();
        }
        
        private void OnAlbumCursorChanged(object o, EventArgs args)
        {
            TreeIter iter;
            
            if(!album_tree.Selection.GetSelected(out iter)) {
                return;
            }
            
            object album_name = album_store.GetValue(iter, 0);
            album_selected = (album_name as string);
            
            TreePath[] treeIndex = album_tree.Selection.GetSelectedRows();
			album_selection_index = treeIndex[0].Indices[0];
             
            if(album_name == null) {
                return;
            }
           
            RunQuery();
        }
        
        private void RunQuery()
        {
        	query = new Select("Tracks");
        	if (artist_selection_index != 0)
        	{
        		query += new Where( 
        		+ new Compare("Artist", Op.EqualTo, artist_selected));
        	}
        	
        	if (album_selection_index != 0)
        	{
        		query += new Where( 
        		+ new Compare("AlbumTitle", Op.EqualTo, album_selected));
        	}
        	
        	if (artist_selection_index != 0 && album_selection_index != 0)
        	{
        		query = new Select("Tracks") + new Where( new Compare("AlbumTitle", Op.EqualTo, album_selected) 
        		+ new And() + new Compare("Artist", Op.EqualTo, artist_selected));	
        	}
        	
        	library_query = query.ToString();
        	
        	EventHandler handler = BrowserClicked;
            	if(handler != null)
             	handler(this, new EventArgs());		
        }
		
	}

}

